searchData={"items":[{"type":"module","title":"Solid","doc":"Solid is an implementation in Elixir of the [Liquid](https://shopify.github.io/liquid/) template language with\nstrict parsing.","ref":"Solid.html"},{"type":"type","title":"Solid.error/0","doc":"","ref":"Solid.html#t:error/0"},{"type":"type","title":"Solid.errors/0","doc":"","ref":"Solid.html#t:errors/0"},{"type":"function","title":"Solid.parse/2","doc":"It generates the compiled template\n\nThis function returns `{:ok, template}` if successfully parses the template, `{:error, template_error}` otherwise\n\n# Options\n\n- `tags` - Override tags allowed during compilation. See `Solid.Tag.default_tags/0` for more information on the default set of tags","ref":"Solid.html#parse/2"},{"type":"function","title":"Solid.parse!/2","doc":"It generates the compiled template\n\nThis function returns the compiled template or raises an error. Same options as `parse/2`","ref":"Solid.html#parse!/2"},{"type":"function","title":"Solid.render/3","doc":"It renders the compiled template using a map with initial vars","ref":"Solid.html#render/3"},{"type":"function","title":"Options - Solid.render/3","doc":"- `file_system`: a tuple of {FileSystemModule, options}. If this option is not specified, `Solid` uses `Solid.BlankFileSystem` which returns an error when the `render` tag is used. `Solid.LocalFileSystem` can be used or a custom module may be implemented. See `Solid.FileSystem` for more details.\n\n- `custom_filters`: a module name where additional filters are defined. The base filters (those from `Solid.StandardFilter`) still can be used, however, custom filters always take precedence.\n\n- `strict_variables`: if `true`, it collects an error when a variable is referenced in the template, but not given in the map\n\n- `strict_filters`: if `true`, it collects an error when a filter is referenced in the template, but not built-in or provided via `custom_filters`\n\n- `matcher_module`: a module to replace `Solid.Matcher` when resolving variables.","ref":"Solid.html#render/3-options"},{"type":"function","title":"Example - Solid.render/3","doc":"fs = Solid.LocalFileSystem.new(\"/path/to/template/dir/\")\nSolid.render(template, vars, [file_system: {Solid.LocalFileSystem, fs}])","ref":"Solid.html#render/3-example"},{"type":"function","title":"Solid.render!/3","doc":"It renders the compiled template using a map with vars\n\nSame options as `render/3`","ref":"Solid.html#render!/3"},{"type":"module","title":"Solid.AccessLiteral","doc":"","ref":"Solid.AccessLiteral.html"},{"type":"type","title":"Solid.AccessLiteral.t/0","doc":"","ref":"Solid.AccessLiteral.html#t:t/0"},{"type":"module","title":"Solid.AccessVariable","doc":"","ref":"Solid.AccessVariable.html"},{"type":"type","title":"Solid.AccessVariable.t/0","doc":"","ref":"Solid.AccessVariable.html#t:t/0"},{"type":"module","title":"Solid.Argument","doc":"","ref":"Solid.Argument.html"},{"type":"function","title":"Solid.Argument.get/4","doc":"","ref":"Solid.Argument.html#get/4"},{"type":"function","title":"Solid.Argument.parse/1","doc":"","ref":"Solid.Argument.html#parse/1"},{"type":"function","title":"Solid.Argument.parse_with_filters/1","doc":"","ref":"Solid.Argument.html#parse_with_filters/1"},{"type":"function","title":"Solid.Argument.render/4","doc":"Similar to get/4 but outputs a printable representation","ref":"Solid.Argument.html#render/4"},{"type":"type","title":"Solid.Argument.t/0","doc":"","ref":"Solid.Argument.html#t:t/0"},{"type":"module","title":"Solid.BinaryCondition","doc":"","ref":"Solid.BinaryCondition.html"},{"type":"function","title":"Solid.BinaryCondition.eval/1","doc":"","ref":"Solid.BinaryCondition.html#eval/1"},{"type":"type","title":"Solid.BinaryCondition.t/0","doc":"","ref":"Solid.BinaryCondition.html#t:t/0"},{"type":"module","title":"Solid.BlankFileSystem","doc":"Default file system that return error on call","ref":"Solid.BlankFileSystem.html"},{"type":"protocol","title":"Solid.Block","doc":"","ref":"Solid.Block.html"},{"type":"function","title":"Solid.Block.blank?/1","doc":"","ref":"Solid.Block.html#blank?/1"},{"type":"type","title":"Solid.Block.t/0","doc":"All the types that implement this protocol.","ref":"Solid.Block.html#t:t/0"},{"type":"behaviour","title":"Solid.Caching","doc":"","ref":"Solid.Caching.html"},{"type":"callback","title":"Solid.Caching.get/1","doc":"","ref":"Solid.Caching.html#c:get/1"},{"type":"callback","title":"Solid.Caching.put/2","doc":"","ref":"Solid.Caching.html#c:put/2"},{"type":"module","title":"Solid.Caching.NoCache","doc":"","ref":"Solid.Caching.NoCache.html"},{"type":"module","title":"Solid.ConditionExpression","doc":"","ref":"Solid.ConditionExpression.html"},{"type":"type","title":"Solid.ConditionExpression.condition/0","doc":"","ref":"Solid.ConditionExpression.html#t:condition/0"},{"type":"function","title":"Solid.ConditionExpression.eval/3","doc":"","ref":"Solid.ConditionExpression.html#eval/3"},{"type":"function","title":"Solid.ConditionExpression.parse/1","doc":"","ref":"Solid.ConditionExpression.html#parse/1"},{"type":"module","title":"Solid.Context","doc":"","ref":"Solid.Context.html"},{"type":"function","title":"Solid.Context.get_counter/2","doc":"","ref":"Solid.Context.html#get_counter/2"},{"type":"function","title":"Solid.Context.get_in/4","doc":"Get data from context respecting the provided scope order.\n\nPossible scope values: :counter_vars, :vars or :iteration_vars","ref":"Solid.Context.html#get_in/4"},{"type":"function","title":"Solid.Context.put_errors/2","doc":"","ref":"Solid.Context.html#put_errors/2"},{"type":"function","title":"Solid.Context.run_cycle/3","doc":"Find the current value that `cycle` must return","ref":"Solid.Context.html#run_cycle/3"},{"type":"type","title":"Solid.Context.scope/0","doc":"","ref":"Solid.Context.html#t:scope/0"},{"type":"type","title":"Solid.Context.t/0","doc":"","ref":"Solid.Context.html#t:t/0"},{"type":"behaviour","title":"Solid.FileSystem","doc":"A file system is a way to let your templates retrieve other templates for use with the include tag.\n\nYou can implement a module that retrieve templates from the database, from the file system using a different path structure, you can provide them as hard-coded inline strings, or any manner that you see fit.\n\nYou can add additional instance variables, arguments, or methods as needed.\n\nExample:\n\n```elixir\nfile_system = Solid.LocalFileSystem.new(template_path)\ntext = Solid.render(template, file_system: {Solid.LocalFileSystem, file_system})\n```\n\nThis will render the template with a LocalFileSystem implementation rooted at 'template_path'.","ref":"Solid.FileSystem.html"},{"type":"callback","title":"Solid.FileSystem.read_template_file/2","doc":"","ref":"Solid.FileSystem.html#c:read_template_file/2"},{"type":"module","title":"Solid.Filter","doc":"","ref":"Solid.Filter.html"},{"type":"type","title":"Solid.Filter.t/0","doc":"","ref":"Solid.Filter.html#t:t/0"},{"type":"module","title":"Solid.HTML","doc":"Conveniences for generating HTML.","ref":"Solid.HTML.html"},{"type":"function","title":"Solid.HTML.html_escape/1","doc":"Escapes the given HTML to string.\n\niex> Plug.HTML.html_escape(\"foo\")\n\"foo\"\n\niex> Plug.HTML.html_escape(\" \")\n\"&lt;foo&gt;\"\n\niex> Plug.HTML.html_escape(\"quotes: \\\" & \\'\")\n\"quotes: &quot; &amp; &#39;\"","ref":"Solid.HTML.html#html_escape/1"},{"type":"function","title":"Solid.HTML.html_escape_to_iodata/1","doc":"Escapes the given HTML to iodata.\n\niex> Plug.HTML.html_escape_to_iodata(\"foo\")\n\"foo\"\n\niex> Plug.HTML.html_escape_to_iodata(\" \")\n[[[] | \"&lt;\"], \"foo\" | \"&gt;\"]\n\niex> Plug.HTML.html_escape_to_iodata(\"quotes: \\\" & \\'\")\n[[[[], \"quotes: \" | \"&quot;\"], \" \" | \"&amp;\"], \" \" | \"&#39;\"]","ref":"Solid.HTML.html#html_escape_to_iodata/1"},{"type":"function","title":"Solid.HTML.replacements/1","doc":"","ref":"Solid.HTML.html#replacements/1"},{"type":"module","title":"Solid.Lexer","doc":"Lexer module for Solid. It is responsible for tokenizing the input text inside an object or a tag.","ref":"Solid.Lexer.html"},{"type":"type","title":"Solid.Lexer.column/0","doc":"","ref":"Solid.Lexer.html#t:column/0"},{"type":"type","title":"Solid.Lexer.line/0","doc":"","ref":"Solid.Lexer.html#t:line/0"},{"type":"type","title":"Solid.Lexer.loc/0","doc":"","ref":"Solid.Lexer.html#t:loc/0"},{"type":"type","title":"Solid.Lexer.operator/0","doc":"","ref":"Solid.Lexer.html#t:operator/0"},{"type":"type","title":"Solid.Lexer.special/0","doc":"","ref":"Solid.Lexer.html#t:special/0"},{"type":"type","title":"Solid.Lexer.token/0","doc":"","ref":"Solid.Lexer.html#t:token/0"},{"type":"function","title":"Solid.Lexer.tokenize_object/1","doc":"Tokenize the input text inside an object","ref":"Solid.Lexer.html#tokenize_object/1"},{"type":"function","title":"Solid.Lexer.tokenize_tag/2","doc":"Tokenize the input text as a complete tag","ref":"Solid.Lexer.html#tokenize_tag/2"},{"type":"function","title":"Solid.Lexer.tokenize_tag_end/1","doc":"Tokenize the rest of the tag after the tag name","ref":"Solid.Lexer.html#tokenize_tag_end/1"},{"type":"function","title":"Solid.Lexer.tokenize_tag_start/2","doc":"Tokenize the tag name for a tag","ref":"Solid.Lexer.html#tokenize_tag_start/2"},{"type":"type","title":"Solid.Lexer.tokens/0","doc":"","ref":"Solid.Lexer.html#t:tokens/0"},{"type":"module","title":"Solid.Literal","doc":"","ref":"Solid.Literal.html"},{"type":"function","title":"Solid.Literal.parse/1","doc":"","ref":"Solid.Literal.html#parse/1"},{"type":"type","title":"Solid.Literal.t/0","doc":"","ref":"Solid.Literal.html#t:t/0"},{"type":"type","title":"Solid.Literal.value/0","doc":"","ref":"Solid.Literal.html#t:value/0"},{"type":"module","title":"Solid.Literal.Empty","doc":"","ref":"Solid.Literal.Empty.html"},{"type":"module","title":"Solid.LocalFileSystem","doc":"This implements an abstract file system which retrieves template files named in a manner similar to Liquid.\nie. with the template name prefixed with an underscore. The extension \".liquid\" is also added.\n\nFor security reasons, template paths are only allowed to contain letters, numbers, and underscore.\n\n**Example:**\n\n    file_system = Solid.LocalFileSystem.new(\"/some/path\")\n\n    Solid.LocalFileSystem.full_path(file_system, \"mypartial\")\n    # => \"/some/path/_mypartial.liquid\"\n\n    Solid.LocalFileSystem.full_path(file_system,\"dir/mypartial\")\n    # => \"/some/path/dir/_mypartial.liquid\"\n\nOptionally in the second argument you can specify a custom pattern for template filenames.\n`%s` will be replaced with template basename\nDefault pattern is \"_%s.liquid\".\n\n**Example:**\n\n    file_system = Solid.LocalFileSystem.new(\"/some/path\", \"%s.html\")\n\n    Solid.LocalFileSystem.full_path( \"index\", file_system)\n    # => \"/some/path/index.html\"","ref":"Solid.LocalFileSystem.html"},{"type":"function","title":"Solid.LocalFileSystem.new/2","doc":"","ref":"Solid.LocalFileSystem.html#new/2"},{"type":"protocol","title":"Solid.Matcher","doc":"","ref":"Solid.Matcher.html"},{"type":"function","title":"Solid.Matcher.match/2","doc":"Assigns context to values","ref":"Solid.Matcher.html#match/2"},{"type":"type","title":"Solid.Matcher.t/0","doc":"All the types that implement this protocol.","ref":"Solid.Matcher.html#t:t/0"},{"type":"module","title":"Solid.Object","doc":"","ref":"Solid.Object.html"},{"type":"function","title":"Solid.Object.parse/1","doc":"","ref":"Solid.Object.html#parse/1"},{"type":"type","title":"Solid.Object.t/0","doc":"","ref":"Solid.Object.html#t:t/0"},{"type":"module","title":"Solid.Parser","doc":"This module contains functions to parse Liquid templates","ref":"Solid.Parser.html"},{"type":"type","title":"Solid.Parser.entry/0","doc":"","ref":"Solid.Parser.html#t:entry/0"},{"type":"type","title":"Solid.Parser.errors/0","doc":"","ref":"Solid.Parser.html#t:errors/0"},{"type":"function","title":"Solid.Parser.maybe_tokenize_tag/2","doc":"Tries to tokenize a tag entry if the tag name is included in the `tag_names`","ref":"Solid.Parser.html#maybe_tokenize_tag/2"},{"type":"function","title":"Solid.Parser.meta_head/1","doc":"Grab the line and column meta information of the first token","ref":"Solid.Parser.html#meta_head/1"},{"type":"function","title":"Solid.Parser.parse/2","doc":"Parse text\n\nIt accepts a :tags option to specify a map of tags to use. See `Solid.Tag.default_tags/0`","ref":"Solid.Parser.html#parse/2"},{"type":"function","title":"Solid.Parser.parse_liquid_entry/1","doc":"Parses a Liquid entry\n\nReturns :ok if there is nothing else to parse.","ref":"Solid.Parser.html#parse_liquid_entry/1"},{"type":"type","title":"Solid.Parser.parse_tree/0","doc":"","ref":"Solid.Parser.html#t:parse_tree/0"},{"type":"function","title":"Solid.Parser.parse_until/3","doc":"Parses a Liquid entry until one of the tags is found\nThe main use case for this function is to parse nested liquid entries\nuntil one of the tags is found. For example an if block:\n\n    {% if true % } Text {{ object }} {% endif %}\n\nusing `parse_until(context, \"endif\", \"Expected endif\")` will parse every liquid entry\nuntil an `endif` tag is found, the end of the template is reached or a parsing error occured.\n\nIf one of the tags is found return what was parsed until such tag was found\nIf none of the tags are found, returns the error tuple with the `reason`","ref":"Solid.Parser.html#parse_until/3"},{"type":"function","title":"Solid.Parser.remove_blank_text_if_blank_body/1","doc":"Remove blank text when the entries account for a \"blank body\". Check `Solid.Block.blank?/1` protocol\nand the implementations","ref":"Solid.Parser.html#remove_blank_text_if_blank_body/1"},{"type":"module","title":"Solid.Parser.Loc","doc":"","ref":"Solid.Parser.Loc.html"},{"type":"type","title":"Solid.Parser.Loc.t/0","doc":"","ref":"Solid.Parser.Loc.html#t:t/0"},{"type":"module","title":"Solid.ParserContext","doc":"","ref":"Solid.ParserContext.html"},{"type":"type","title":"Solid.ParserContext.t/0","doc":"","ref":"Solid.ParserContext.html#t:t/0"},{"type":"module","title":"Solid.Range","doc":"Range representation\n\n(first..last)\n(1..5)","ref":"Solid.Range.html"},{"type":"function","title":"Solid.Range.parse/1","doc":"","ref":"Solid.Range.html#parse/1"},{"type":"type","title":"Solid.Range.t/0","doc":"","ref":"Solid.Range.html#t:t/0"},{"type":"protocol","title":"Solid.Renderable","doc":"","ref":"Solid.Renderable.html"},{"type":"function","title":"Solid.Renderable.render/3","doc":"","ref":"Solid.Renderable.html#render/3"},{"type":"type","title":"Solid.Renderable.t/0","doc":"All the types that implement this protocol.","ref":"Solid.Renderable.html#t:t/0"},{"type":"module","title":"Solid.Sigil","doc":"Provides the `~LIQUID` sigil for validating and compiling Liquid templates using Solid.\n\nThis sigil validates the template at compile time and returns a compiled Solid template.\nIf the template has syntax errors, it will raise a CompileError with detailed information.","ref":"Solid.Sigil.html"},{"type":"module","title":"Examples - Solid.Sigil","doc":"iex> import Solid.Sigil\n    iex> template = ~LIQUID\"\"\"\n    ...> Hello, {{ name }}!\n    ...> \"\"\"\n    iex> Solid.render(template, %{\"name\" => \"World\"})\n    {:ok, \"Hello, World!\"}","ref":"Solid.Sigil.html#module-examples"},{"type":"macro","title":"Solid.Sigil.sigil_LIQUID/2","doc":"","ref":"Solid.Sigil.html#sigil_LIQUID/2"},{"type":"module","title":"Solid.StandardFilter","doc":"Standard filters","ref":"Solid.StandardFilter.html"},{"type":"function","title":"Solid.StandardFilter.abs/1","doc":"Returns the absolute value of a number.\n\niex> Solid.StandardFilter.abs(-17)\n17\niex> Solid.StandardFilter.abs(17)\n17\niex> Solid.StandardFilter.abs(\"-17.5\")\n17.5","ref":"Solid.StandardFilter.html#abs/1"},{"type":"function","title":"Solid.StandardFilter.append/2","doc":"Concatenates two strings and returns the concatenated value.\n\niex> Solid.StandardFilter.append(\"www.example.com\", \"/index.html\")\n\"www.example.com/index.html\"","ref":"Solid.StandardFilter.html#append/2"},{"type":"function","title":"Solid.StandardFilter.apply/4","doc":"","ref":"Solid.StandardFilter.html#apply/4"},{"type":"function","title":"Solid.StandardFilter.at_least/2","doc":"Limits a number to a minimum value.\n\niex> Solid.StandardFilter.at_least(5, 3)\n5\niex> Solid.StandardFilter.at_least(2, 4)\n4","ref":"Solid.StandardFilter.html#at_least/2"},{"type":"function","title":"Solid.StandardFilter.at_most/2","doc":"Limits a number to a maximum value.\n\niex> Solid.StandardFilter.at_most(5, 3)\n3\niex> Solid.StandardFilter.at_most(2, 4)\n2","ref":"Solid.StandardFilter.html#at_most/2"},{"type":"function","title":"Solid.StandardFilter.base64_decode/1","doc":"Decodes a string in Base64 format.\n\niex> Solid.StandardFilter.base64_decode(\"YXBwbGVz\")\n\"apples\"","ref":"Solid.StandardFilter.html#base64_decode/1"},{"type":"function","title":"Solid.StandardFilter.base64_encode/1","doc":"Encodes a string to Base64 format.\n\niex> Solid.StandardFilter.base64_encode(\"apples\")\n\"YXBwbGVz\"\niex> Solid.StandardFilter.base64_encode(3)\n\"Mw==\"","ref":"Solid.StandardFilter.html#base64_encode/1"},{"type":"function","title":"Solid.StandardFilter.base64_url_safe_decode/1","doc":"Decodes a string in URL-safe Base64 format.\n\niex> Solid.StandardFilter.base64_url_safe_decode(\"YXBwbGVz\")\n\"apples\"","ref":"Solid.StandardFilter.html#base64_url_safe_decode/1"},{"type":"function","title":"Solid.StandardFilter.base64_url_safe_encode/1","doc":"Encodes a string to URL-safe Base64 format.\n\niex> Solid.StandardFilter.base64_url_safe_encode(\"apples\")\n\"YXBwbGVz\"\n\niex> Solid.StandardFilter.base64_url_safe_encode(3)\n\"Mw==\"","ref":"Solid.StandardFilter.html#base64_url_safe_encode/1"},{"type":"function","title":"Solid.StandardFilter.capitalize/1","doc":"Makes the first character of a string capitalized.\n\niex> Solid.StandardFilter.capitalize(\"my great title\")\n\"My great title\"\niex> Solid.StandardFilter.capitalize(1)\n\"1\"","ref":"Solid.StandardFilter.html#capitalize/1"},{"type":"function","title":"Solid.StandardFilter.ceil/1","doc":"Rounds the input up to the nearest whole number. Liquid tries to convert the input to a number before the filter is applied.","ref":"Solid.StandardFilter.html#ceil/1"},{"type":"function","title":"Solid.StandardFilter.compact/1","doc":"Removes all occurrences of nil from a list\n\niex> Solid.StandardFilter.compact([1, nil, 2, nil, 3])\n[1, 2, 3]","ref":"Solid.StandardFilter.html#compact/1"},{"type":"function","title":"Solid.StandardFilter.compact/2","doc":"","ref":"Solid.StandardFilter.html#compact/2"},{"type":"function","title":"Solid.StandardFilter.concat/2","doc":"Concatenates (joins together) multiple arrays.\nThe resulting array contains all the items from the input arrays.\n\niex> Solid.StandardFilter.concat([1, 2], [3, 4])\n[1, 2, 3, 4]\n\niex> Solid.StandardFilter.concat(nil, [3, 4])\n[3, 4]","ref":"Solid.StandardFilter.html#concat/2"},{"type":"function","title":"Solid.StandardFilter.date/2","doc":"Converts a `DateTime`/`NaiveDateTime` struct into another date format.\nThe input may also be a Unix timestamp or an ISO 8601 date string.\n\nThe format for this syntax is the same as `Calendar.strftime/2`.\n\nTo get the current time, pass the special word `\"now\"` (or `\"today\"`) to `date`.\n\niex> Solid.StandardFilter.date(\"1970-01-01 00:00:00Z\", \"%s\")\n\"0\"\niex> Solid.StandardFilter.date(\"1970-01-01 00:00:01Z\", \"%s\")\n\"1\"","ref":"Solid.StandardFilter.html#date/2"},{"type":"function","title":"Solid.StandardFilter.default/3","doc":"Allows you to specify a fallback in case a value doesn’t exist.\n`default` will show its value if the left side is nil, false, or empty\n\niex> Solid.StandardFilter.default(123, 456)\n123\n\niex> Solid.StandardFilter.default(nil, 456)\n456\n\niex> Solid.StandardFilter.default(false, 456)\n456\n\niex> Solid.StandardFilter.default([], 456)\n456\n\niex> Solid.StandardFilter.default(\"\", 456)\n456","ref":"Solid.StandardFilter.html#default/3"},{"type":"function","title":"Solid.StandardFilter.divided_by/2","doc":"Divides a number by the specified number.\n\nThe result is rounded down to the nearest integer (that is, the floor) if the divisor is an integer.\n\n{{ 16 | divided_by: 4 }}\niex> Solid.StandardFilter.divided_by(16, 4)\n4\niex> Solid.StandardFilter.divided_by(5, 3)\n1\niex> Solid.StandardFilter.divided_by(20, 7)\n2","ref":"Solid.StandardFilter.html#divided_by/2"},{"type":"function","title":"Solid.StandardFilter.downcase/1","doc":"Makes each character in a string lowercase.\nIt has no effect on strings which are already all lowercase.\n\niex> Solid.StandardFilter.downcase(\"aBc\")\n\"abc\"\n\niex> Solid.StandardFilter.downcase(456)\n\"456\"\n\niex> Solid.StandardFilter.downcase(nil)\n\"\"","ref":"Solid.StandardFilter.html#downcase/1"},{"type":"function","title":"Solid.StandardFilter.escape/1","doc":"HTML encodes the string.\n\nOutput\niex> Solid.StandardFilter.escape(\"Have you read 'James & the Giant Peach'?\")\n\"Have you read &#39;James &amp; the Giant Peach&#39;?\"","ref":"Solid.StandardFilter.html#escape/1"},{"type":"function","title":"Solid.StandardFilter.escape_once/1","doc":"HTML encodes the string without encoding already encoded characters again.\n\nThis mimics the regex based approach of the ruby library.\n\nOutput\n\"1 &lt; 2 &amp; 3\"\n\niex> Solid.StandardFilter.escape_once(\"1 &lt; 2 &amp; 3\")\n\"1 &lt; 2 &amp; 3\"","ref":"Solid.StandardFilter.html#escape_once/1"},{"type":"function","title":"Solid.StandardFilter.first/1","doc":"Returns the first item of an array.\n\niex> Solid.StandardFilter.first([1, 2, 3])\n1\niex> Solid.StandardFilter.first([])\nnil\niex> Solid.StandardFilter.first(%{\"a\" => \"b\"})\n[\"a\", \"b\"]","ref":"Solid.StandardFilter.html#first/1"},{"type":"function","title":"Solid.StandardFilter.floor/1","doc":"Rounds a number down to the nearest whole number.\nSolid tries to convert the input to a number before the filter is applied.\n\niex> Solid.StandardFilter.floor(1.2)\n1\niex> Solid.StandardFilter.floor(2.0)\n2\niex> Solid.StandardFilter.floor(\"3.5\")\n3","ref":"Solid.StandardFilter.html#floor/1"},{"type":"function","title":"Solid.StandardFilter.join/2","doc":"Join a list of strings returning one String glued by `glue`\n\niex> Solid.StandardFilter.join([\"a\", \"b\", \"c\"])\n\"a b c\"\niex> Solid.StandardFilter.join([\"a\", \"b\", \"c\"], \"-\")\n\"a-b-c\"\niex> Solid.StandardFilter.join([\"a\", \"b\", \"c\"], 5)\n\"a5b5c\"\niex> Solid.StandardFilter.join((0..3), \"-\")\n\"0-1-2-3\"\niex> Solid.StandardFilter.join((3..0//-1), \"-\")\n\"3-2-1-0\"\niex> Solid.StandardFilter.join(5, \"-\")\n5","ref":"Solid.StandardFilter.html#join/2"},{"type":"function","title":"Solid.StandardFilter.last/1","doc":"Returns the last item of an array.\n\niex> Solid.StandardFilter.last([1, 2, 3])\n3\niex> Solid.StandardFilter.last([])\nnil","ref":"Solid.StandardFilter.html#last/1"},{"type":"function","title":"Solid.StandardFilter.lstrip/1","doc":"Removes all whitespaces (tabs, spaces, and newlines) from the beginning of a string.\nThe filter does not affect spaces between words.\n\niex> Solid.StandardFilter.lstrip(\"          So much room for activities!          \")\n\"So much room for activities!          \"","ref":"Solid.StandardFilter.html#lstrip/1"},{"type":"function","title":"Solid.StandardFilter.map/2","doc":"Map through a list of hashes accessing `property`\n\niex> Solid.StandardFilter.map([%{\"a\" => \"A\"}, %{\"a\" => 1}], \"a\")\n[\"A\", 1]","ref":"Solid.StandardFilter.html#map/2"},{"type":"function","title":"Solid.StandardFilter.minus/2","doc":"Subtracts a number from another number.\n\niex> Solid.StandardFilter.minus(4, 2)\n2\niex> Solid.StandardFilter.minus(16, 4)\n12\niex> Solid.StandardFilter.minus(183.357, 12)\n171.357","ref":"Solid.StandardFilter.html#minus/2"},{"type":"function","title":"Solid.StandardFilter.modulo/2","doc":"Subtracts a number from another number.\n\niex> Solid.StandardFilter.modulo(3, 2)\n1\niex> Solid.StandardFilter.modulo(24, 7)\n3\niex> Solid.StandardFilter.modulo(183.357, 12)\n3.357","ref":"Solid.StandardFilter.html#modulo/2"},{"type":"function","title":"Solid.StandardFilter.newline_to_br/1","doc":"Replaces every newline in a string with an HTML line break ( ).\n\nOutput\niex> Solid.StandardFilter.newline_to_br(\"Test \\ntext\\r\\n with line breaks.\")\n\"Test  \\ntext \\r\\n with line breaks.\"\n\niex> Solid.StandardFilter.newline_to_br([[[\"Test \\ntext\\r\\n with \"] | \"line breaks.\"]])\n\"Test  \\ntext \\r\\n with line breaks.\"","ref":"Solid.StandardFilter.html#newline_to_br/1"},{"type":"function","title":"Solid.StandardFilter.plus/2","doc":"Adds a number to another number.\n\niex> Solid.StandardFilter.plus(4, 2)\n6\niex> Solid.StandardFilter.plus(16, 4)\n20\niex> Solid.StandardFilter.plus(\"16\", 4)\n20\niex> Solid.StandardFilter.plus(183.357, 12)\n195.357\niex> Solid.StandardFilter.plus(\"183.357\", 12)\n195.357\niex> Solid.StandardFilter.plus(\"183.ABC357\", 12)\n195","ref":"Solid.StandardFilter.html#plus/2"},{"type":"function","title":"Solid.StandardFilter.prepend/2","doc":"Adds the specified string to the beginning of another string.\n\niex> Solid.StandardFilter.prepend(\"/index.html\", \"www.example.com\")\n\"www.example.com/index.html\"","ref":"Solid.StandardFilter.html#prepend/2"},{"type":"function","title":"Solid.StandardFilter.remove/2","doc":"Removes every occurrence of the specified substring from a string.\n\niex> Solid.StandardFilter.remove(\"I strained to see the train through the rain\", \"rain\")\n\"I sted to see the t through the \"","ref":"Solid.StandardFilter.html#remove/2"},{"type":"function","title":"Solid.StandardFilter.remove_first/2","doc":"Removes only the first occurrence of the specified substring from a string.\n\niex> Solid.StandardFilter.remove_first(\"I strained to see the train through the rain\", \"rain\")\n\"I sted to see the train through the rain\"","ref":"Solid.StandardFilter.html#remove_first/2"},{"type":"function","title":"Solid.StandardFilter.remove_last/2","doc":"Removes only the last occurrence of the specified substring from a string.\n\niex> Solid.StandardFilter.remove_last(\"I strained to see the train through the rain\", \"rain\")\n\"I strained to see the train through the \"","ref":"Solid.StandardFilter.html#remove_last/2"},{"type":"function","title":"Solid.StandardFilter.replace/3","doc":"Replaces every occurrence of an argument in a string with the second argument.\n\niex> Solid.StandardFilter.replace(\"Take my protein pills and put my helmet on\", \"my\", \"your\")\n\"Take your protein pills and put your helmet on\"","ref":"Solid.StandardFilter.html#replace/3"},{"type":"function","title":"Solid.StandardFilter.replace_first/3","doc":"Replaces only the first occurrence of the first argument in a string with the second argument.\n\niex> Solid.StandardFilter.replace_first(\"Take my protein pills and put my helmet on\", \"my\", \"your\")\n\"Take your protein pills and put my helmet on\"","ref":"Solid.StandardFilter.html#replace_first/3"},{"type":"function","title":"Solid.StandardFilter.replace_last/3","doc":"Replaces only the last occurrence of the first argument in a string with the second argument.\n\niex> Solid.StandardFilter.replace_last(\"Take my protein pills and put my helmet on\", \"my\", \"your\")\n\"Take my protein pills and put your helmet on\"","ref":"Solid.StandardFilter.html#replace_last/3"},{"type":"function","title":"Solid.StandardFilter.reverse/1","doc":"Reverses the order of the items in an array. reverse cannot reverse a string.\n\niex> Solid.StandardFilter.reverse([\"a\", \"b\", \"c\"])\n[\"c\", \"b\", \"a\"]","ref":"Solid.StandardFilter.html#reverse/1"},{"type":"function","title":"Solid.StandardFilter.round/2","doc":"Rounds an input number to the nearest integer or,\nif a number is specified as an argument, to that number of decimal places.\n\niex> Solid.StandardFilter.round(1.2)\n1\niex> Solid.StandardFilter.round(2.7)\n3\niex> Solid.StandardFilter.round(183.357, 2)\n183.36","ref":"Solid.StandardFilter.html#round/2"},{"type":"function","title":"Solid.StandardFilter.rstrip/1","doc":"Removes all whitespace (tabs, spaces, and newlines) from the right side of a string.\n\niex> Solid.StandardFilter.rstrip(\"          So much room for activities!          \")\n\"          So much room for activities!\"","ref":"Solid.StandardFilter.html#rstrip/1"},{"type":"function","title":"Solid.StandardFilter.size/1","doc":"Returns the number of characters in a string or the number of items in an array.\n\niex> Solid.StandardFilter.size(\"Ground control to Major Tom.\")\n28\niex> Solid.StandardFilter.size(~w(ground control to Major Tom.))\n5","ref":"Solid.StandardFilter.html#size/1"},{"type":"function","title":"Solid.StandardFilter.slice/3","doc":"Returns a substring of 1 character beginning at the index specified by the argument passed in.\nAn optional second argument specifies the length of the substring to be returned.\n\nString indices are numbered starting from 0.\n\niex> Solid.StandardFilter.slice(\"Liquid\", 0)\n\"L\"\n\niex> Solid.StandardFilter.slice(\"Liquid\", 2)\n\"q\"\n\niex> Solid.StandardFilter.slice(\"Liquid\", 2, 5)\n\"quid\"\niex> Solid.StandardFilter.slice(\"Liquid\", -3, 2)\n\"ui\"","ref":"Solid.StandardFilter.html#slice/3"},{"type":"function","title":"Solid.StandardFilter.sort/1","doc":"Sorts items in an array by a property of an item in the array. The order of the sorted array is case-sensitive.\n\niex> Solid.StandardFilter.sort(~w(zebra octopus giraffe SallySnake))\n~w(SallySnake giraffe octopus zebra)","ref":"Solid.StandardFilter.html#sort/1"},{"type":"function","title":"Solid.StandardFilter.sort_natural/1","doc":"Sorts items in an array by a property of an item in the array. The order of the sorted array is case-sensitive.\n\niex> Solid.StandardFilter.sort_natural(~w(zebra octopus giraffe SallySnake))\n~w(giraffe octopus SallySnake zebra)","ref":"Solid.StandardFilter.html#sort_natural/1"},{"type":"function","title":"Solid.StandardFilter.split/2","doc":"Split input string into an array of substrings separated by given pattern.\n\niex> Solid.StandardFilter.split(\"a b c\", \" \")\n~w(a b c)\niex> Solid.StandardFilter.split(\"\", \" \")\n[\"\"]","ref":"Solid.StandardFilter.html#split/2"},{"type":"function","title":"Solid.StandardFilter.strip/1","doc":"Removes all whitespace (tabs, spaces, and newlines) from both the left and right side of a string.\nIt does not affect spaces between words.\n\niex> Solid.StandardFilter.strip(\"          So much room for activities!          \")\n\"So much room for activities!\"","ref":"Solid.StandardFilter.html#strip/1"},{"type":"function","title":"Solid.StandardFilter.strip_html/1","doc":"Removes any HTML tags from a string.\n\nThis mimics the regex based approach of the ruby library.\n\nOutput\niex> Solid.StandardFilter.strip_html(\"Have  you  read  Ulysses ?\")\n\"Have you read Ulysses?\"","ref":"Solid.StandardFilter.html#strip_html/1"},{"type":"function","title":"Solid.StandardFilter.strip_newlines/1","doc":"Removes any newline characters (line breaks) from a string.\n\nOutput\niex> Solid.StandardFilter.strip_newlines(\"Test \\ntext\\r\\n with line breaks.\")\n\"Test text with line breaks.\"\n\niex> Solid.StandardFilter.strip_newlines([[[\"Test \\ntext\\r\\n with \"] | \"line breaks.\"]])\n\"Test text with line breaks.\"","ref":"Solid.StandardFilter.html#strip_newlines/1"},{"type":"function","title":"Solid.StandardFilter.times/2","doc":"Multiplies a number by another number.\n\niex> Solid.StandardFilter.times(3, 2)\n6\niex> Solid.StandardFilter.times(24, 7)\n168\niex> Solid.StandardFilter.times(183.357, 12)\n2200.284","ref":"Solid.StandardFilter.html#times/2"},{"type":"function","title":"Solid.StandardFilter.truncate/3","doc":"truncate shortens a string down to the number of characters passed as a parameter.\nIf the number of characters specified is less than the length of the string, an ellipsis (…) is appended to the string\nand is included in the character count.\n\niex> Solid.StandardFilter.truncate(\"Ground control to Major Tom.\", 20)\n\"Ground control to...\"\n\n# Custom ellipsis\n\ntruncate takes an optional second parameter that specifies the sequence of characters to be appended to the truncated string.\nBy default this is an ellipsis (…), but you can specify a different sequence.\n\nThe length of the second parameter counts against the number of characters specified by the first parameter.\nFor example, if you want to truncate a string to exactly 10 characters, and use a 3-character ellipsis,\nuse 13 for the first parameter of truncate, since the ellipsis counts as 3 characters.\n\niex> Solid.StandardFilter.truncate(\"Ground control to Major Tom.\", 25, \", and so on\")\n\"Ground control, and so on\"\n\n# No ellipsis\n\nYou can truncate to the exact number of characters specified by the first parameter\nand show no trailing characters by passing a blank string as the second parameter:\n\niex> Solid.StandardFilter.truncate(\"Ground control to Major Tom.\", 20, \"\")\n\"Ground control to Ma\"","ref":"Solid.StandardFilter.html#truncate/3"},{"type":"function","title":"Solid.StandardFilter.truncatewords/3","doc":"Shortens a string down to the number of words passed as the argument.\nIf the specified number of words is less than the number of words in the string, an ellipsis (…) is appended to the string.\n\niex> Solid.StandardFilter.truncatewords(\"Ground control to Major Tom.\", 3)\n\"Ground control to...\"\n\n# Custom ellipsis\n\n`truncatewords` takes an optional second parameter that specifies the sequence of characters to be appended to the truncated string.\nBy default this is an ellipsis (…), but you can specify a different sequence.\n\niex> Solid.StandardFilter.truncatewords(\"Ground control to Major Tom.\", 3, \"--\")\n\"Ground control to--\"\n\n# No ellipsis\n\nYou can avoid showing trailing characters by passing a blank string as the second parameter:\n\niex> Solid.StandardFilter.truncatewords(\"Ground control to Major Tom.\", 3, \"\")\n\"Ground control to\"","ref":"Solid.StandardFilter.html#truncatewords/3"},{"type":"function","title":"Solid.StandardFilter.uniq/1","doc":"Removes any duplicate elements in an array.\n\nOutput\niex> Solid.StandardFilter.uniq(~w(ants bugs bees bugs ants))\n~w(ants bugs bees)","ref":"Solid.StandardFilter.html#uniq/1"},{"type":"function","title":"Solid.StandardFilter.upcase/1","doc":"Makes each character in a string uppercase.\nIt has no effect on strings which are already all uppercase.\n\niex> Solid.StandardFilter.upcase(\"aBc\")\n\"ABC\"\n\niex> Solid.StandardFilter.upcase(456)\n\"456\"\n\niex> Solid.StandardFilter.upcase(nil)\n\"\"","ref":"Solid.StandardFilter.html#upcase/1"},{"type":"function","title":"Solid.StandardFilter.url_decode/1","doc":"URL decodes the string.\n\nOutput\niex> Solid.StandardFilter.url_decode(\"%27Stop%21%27+said+Fred\")\n\"'Stop!' said Fred\"","ref":"Solid.StandardFilter.html#url_decode/1"},{"type":"function","title":"Solid.StandardFilter.url_encode/1","doc":"URL encodes the string.\n\nOutput\niex> Solid.StandardFilter.url_encode(\"john@liquid.com\")\n\"john%40liquid.com\"\n\niex> Solid.StandardFilter.url_encode(\"Tetsuro Takara\")\n\"Tetsuro+Takara\"","ref":"Solid.StandardFilter.html#url_encode/1"},{"type":"function","title":"Solid.StandardFilter.where/2","doc":"","ref":"Solid.StandardFilter.html#where/2"},{"type":"function","title":"Solid.StandardFilter.where/3","doc":"Creates an array including only the objects with a given property value,\nor any truthy value by default.\n\nOutput\niex> input = [\n...>   %{\"id\" => 1, \"type\" => \"kitchen\"},\n...>   %{\"id\" => 2, \"type\" => \"bath\"},\n...>   %{\"id\" => 3, \"type\" => \"kitchen\"}\n...> ]\niex> Solid.StandardFilter.where(input, \"type\", \"kitchen\")\n[%{\"id\" => 1, \"type\" => \"kitchen\"}, %{\"id\" => 3, \"type\" => \"kitchen\"}]\n\niex> input = [\n...>   %{\"id\" => 1, \"available\" => true},\n...>   %{\"id\" => 2, \"type\" => false},\n...>   %{\"id\" => 3, \"available\" => true}\n...> ]\niex> Solid.StandardFilter.where(input, \"available\")\n[%{\"id\" => 1, \"available\" => true}, %{\"id\" => 3, \"available\" => true}]","ref":"Solid.StandardFilter.html#where/3"},{"type":"behaviour","title":"Solid.Tag","doc":"","ref":"Solid.Tag.html"},{"type":"function","title":"Solid.Tag.default_tags/0","doc":"","ref":"Solid.Tag.html#default_tags/0"},{"type":"function","title":"Solid.Tag.parse/3","doc":"","ref":"Solid.Tag.html#parse/3"},{"type":"callback","title":"Solid.Tag.parse/3","doc":"","ref":"Solid.Tag.html#c:parse/3"},{"type":"module","title":"Solid.Tags.AssignTag","doc":"","ref":"Solid.Tags.AssignTag.html"},{"type":"type","title":"Solid.Tags.AssignTag.t/0","doc":"","ref":"Solid.Tags.AssignTag.html#t:t/0"},{"type":"module","title":"Solid.Tags.BreakTag","doc":"","ref":"Solid.Tags.BreakTag.html"},{"type":"module","title":"Solid.Tags.CaptureTag","doc":"","ref":"Solid.Tags.CaptureTag.html"},{"type":"type","title":"Solid.Tags.CaptureTag.t/0","doc":"","ref":"Solid.Tags.CaptureTag.html#t:t/0"},{"type":"module","title":"Solid.Tags.CaseTag","doc":"","ref":"Solid.Tags.CaseTag.html"},{"type":"type","title":"Solid.Tags.CaseTag.t/0","doc":"","ref":"Solid.Tags.CaseTag.html#t:t/0"},{"type":"module","title":"Solid.Tags.CommentTag","doc":"","ref":"Solid.Tags.CommentTag.html"},{"type":"module","title":"Solid.Tags.ContinueTag","doc":"","ref":"Solid.Tags.ContinueTag.html"},{"type":"module","title":"Solid.Tags.CounterTag","doc":"","ref":"Solid.Tags.CounterTag.html"},{"type":"type","title":"Solid.Tags.CounterTag.t/0","doc":"","ref":"Solid.Tags.CounterTag.html#t:t/0"},{"type":"module","title":"Solid.Tags.CycleTag","doc":"","ref":"Solid.Tags.CycleTag.html"},{"type":"type","title":"Solid.Tags.CycleTag.t/0","doc":"","ref":"Solid.Tags.CycleTag.html#t:t/0"},{"type":"module","title":"Solid.Tags.EchoTag","doc":"","ref":"Solid.Tags.EchoTag.html"},{"type":"module","title":"Solid.Tags.ForTag","doc":"","ref":"Solid.Tags.ForTag.html"},{"type":"type","title":"Solid.Tags.ForTag.t/0","doc":"","ref":"Solid.Tags.ForTag.html#t:t/0"},{"type":"module","title":"Solid.Tags.IfTag","doc":"Handle if and unless tags","ref":"Solid.Tags.IfTag.html"},{"type":"type","title":"Solid.Tags.IfTag.t/0","doc":"","ref":"Solid.Tags.IfTag.html#t:t/0"},{"type":"module","title":"Solid.Tags.InlineCommentTag","doc":"","ref":"Solid.Tags.InlineCommentTag.html"},{"type":"module","title":"Solid.Tags.RawTag","doc":"","ref":"Solid.Tags.RawTag.html"},{"type":"module","title":"Solid.Tags.RenderTag","doc":"","ref":"Solid.Tags.RenderTag.html"},{"type":"type","title":"Solid.Tags.RenderTag.t/0","doc":"","ref":"Solid.Tags.RenderTag.html#t:t/0"},{"type":"module","title":"Solid.Template","doc":"","ref":"Solid.Template.html"},{"type":"type","title":"Solid.Template.t/0","doc":"","ref":"Solid.Template.html#t:t/0"},{"type":"module","title":"Solid.Text","doc":"","ref":"Solid.Text.html"},{"type":"type","title":"Solid.Text.t/0","doc":"","ref":"Solid.Text.html#t:t/0"},{"type":"module","title":"Solid.UnaryCondition","doc":"","ref":"Solid.UnaryCondition.html"},{"type":"function","title":"Solid.UnaryCondition.eval/1","doc":"","ref":"Solid.UnaryCondition.html#eval/1"},{"type":"type","title":"Solid.UnaryCondition.t/0","doc":"","ref":"Solid.UnaryCondition.html#t:t/0"},{"type":"module","title":"Solid.Variable","doc":"","ref":"Solid.Variable.html"},{"type":"type","title":"Solid.Variable.accesses/0","doc":"","ref":"Solid.Variable.html#t:accesses/0"},{"type":"function","title":"Solid.Variable.parse/1","doc":"","ref":"Solid.Variable.html#parse/1"},{"type":"type","title":"Solid.Variable.t/0","doc":"","ref":"Solid.Variable.html#t:t/0"},{"type":"exception","title":"Solid.ArgumentError","doc":"","ref":"Solid.ArgumentError.html"},{"type":"type","title":"Solid.ArgumentError.t/0","doc":"","ref":"Solid.ArgumentError.html#t:t/0"},{"type":"exception","title":"Solid.FileSystem.Error","doc":"","ref":"Solid.FileSystem.Error.html"},{"type":"function","title":"Solid.FileSystem.Error.message/1","doc":"","ref":"Solid.FileSystem.Error.html#message/1"},{"type":"type","title":"Solid.FileSystem.Error.t/0","doc":"","ref":"Solid.FileSystem.Error.html#t:t/0"},{"type":"exception","title":"Solid.ParserError","doc":"","ref":"Solid.ParserError.html"},{"type":"type","title":"Solid.ParserError.t/0","doc":"","ref":"Solid.ParserError.html#t:t/0"},{"type":"exception","title":"Solid.RenderError","doc":"","ref":"Solid.RenderError.html"},{"type":"type","title":"Solid.RenderError.t/0","doc":"","ref":"Solid.RenderError.html#t:t/0"},{"type":"exception","title":"Solid.TemplateError","doc":"","ref":"Solid.TemplateError.html"},{"type":"type","title":"Solid.TemplateError.t/0","doc":"","ref":"Solid.TemplateError.html#t:t/0"},{"type":"exception","title":"Solid.UndefinedFilterError","doc":"","ref":"Solid.UndefinedFilterError.html"},{"type":"type","title":"Solid.UndefinedFilterError.t/0","doc":"","ref":"Solid.UndefinedFilterError.html#t:t/0"},{"type":"exception","title":"Solid.UndefinedVariableError","doc":"","ref":"Solid.UndefinedVariableError.html"},{"type":"type","title":"Solid.UndefinedVariableError.t/0","doc":"","ref":"Solid.UndefinedVariableError.html#t:t/0"},{"type":"exception","title":"Solid.WrongFilterArityError","doc":"","ref":"Solid.WrongFilterArityError.html"},{"type":"type","title":"Solid.WrongFilterArityError.t/0","doc":"","ref":"Solid.WrongFilterArityError.html#t:t/0"},{"type":"extras","title":"Contributing","doc":"# Contributing to Solid\n\nThanks for taking the time to help us build Solid 🎉\n\nThe most important part of our test suite is the directory `test/solid/integration/scenarios`.\nWe have `input.liquid` files that are tested against Solid and against the Liquid gem.\n\nWhenever possible we want to include a new test case that covers the new feature that was added or bug that has been fixed. So if it makes sense please include a new test case or change an existing one that makes sense.","ref":"contributing.html"},{"type":"extras","title":"License","doc":"# MIT License\n\nCopyright (c) 2016 Eduardo Gurgel Pinho\n\nPermission is hereby granted, free of charge, to any person obtaining\na copy of this software and associated documentation files (the\n\"Software\"), to deal in the Software without restriction, including\nwithout limitation the rights to use, copy, modify, merge, publish,\ndistribute, sublicense, and/or sell copies of the Software, and to\npermit persons to whom the Software is furnished to do so, subject to\nthe following conditions:\n\nThe above copyright notice and this permission notice shall be\nincluded in all copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND,\nEXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF\nMERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND\nNONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE\nLIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION\nOF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION\nWITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html"},{"type":"extras","title":"Overview","doc":"# Solid\n[![Module Version](https://img.shields.io/hexpm/v/solid.svg)](https://hex.pm/packages/solid)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/solid/)\n[![Total Download](https://img.shields.io/hexpm/dt/solid.svg)](https://hex.pm/packages/solid)\n[![License](https://img.shields.io/hexpm/l/solid.svg)](https://github.com/edgurgel/solid/blob/master/LICENSE.md)\n[![Last Updated](https://img.shields.io/github/last-commit/edgurgel/solid.svg)](https://github.com/edgurgel/solid/commits/master)\n\nSolid is an implementation in Elixir of the [Liquid](https://shopify.github.io/liquid/) template language with strict parsing.","ref":"readme.html"},{"type":"extras","title":"Basic Usage - Overview","doc":"```elixir\niex> template = \"My name is {{ user.name }}\"\niex> {:ok, template} = Solid.parse(template)\niex> Solid.render!(template, %{ \"user\" => %{ \"name\" => \"José\" } }) |> to_string\n\"My name is José\"\n```","ref":"readme.html#basic-usage"},{"type":"extras","title":"Installation - Overview","doc":"The package can be installed with:\n\n```elixir\ndef deps do\n  [{:solid, \"~> 1.0.0-rc.0\"}]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Custom tags - Overview","doc":"To implement a new tag you need to create a new module that implements the `Tag` behaviour. It must implement a `parse/3` function that returns a struct that implements `Solid.Renderable`. Here is a simple example:\n\n```elixir\ndefmodule CurrentYear do\n  @enforce_keys [:loc]\n  defstruct [:loc]\n\n  @behaviour Solid.Tag\n\n  @impl true\n  def parse(\"get_current_year\", loc, context) do\n    with {:ok, [{:end, _}], context} <- Solid.Lexer.tokenize_tag_end(context) do\n      {:ok, %__MODULE__{loc: loc}, context}\n    end\n  end\n\n  defimpl Solid.Renderable do\n    def render(_tag, context, _options) do\n      {[to_string(Date.utc_today().year)], context}\n    end\n  end\nend\n```\n\nNow to use it simply pass a `:tags` option to `Solid.parse/2` including your custom tag:\n\n```elixir\ntags = Map.put(Solid.Tag.default_tags(), \"get_current_year\", CurrentYear)\nSolid.parse!(\"{{ get_current_year }}\", tags: tags)\n```\n\nOne can also pass a subset of the default tags if a more restricted environment is necessary:\n\n```elixir\n# No comment tags allowed\ntags = Map.delete(Solid.Tag.default_tags(), \"comment\")\nSolid.parse!(\"{% comment %} {% endcomment %}\", tags: tags)\n```\n\nAn error will be presented as `comment` is not part of the allowed tags:\n\n```elixir\n** (Solid.TemplateError) Unexpected tag 'comment'\n1: {% comment %} {% endcomment %}\n   ^\nUnexpected tag 'endcomment'\n1: {% comment %} {% endcomment %}\n                 ^\n    (solid 1.0.0-rc1) lib/solid.ex:51: Solid.parse!/2\n    iex:2: (file)\n```","ref":"readme.html#custom-tags"},{"type":"extras","title":"Custom filters - Overview","doc":"While calling `Solid.render` one can pass a module with custom filters:\n\n```elixir\ndefmodule MyCustomFilters do\n  def add_one(x), do: x + 1\nend\n\n\"{{ number | add_one }}\"\n|> Solid.parse!()\n|> Solid.render!(%{ \"number\" => 41}, custom_filters: MyCustomFilters)\n|> IO.puts()\n# 42\n```","ref":"readme.html#custom-filters"},{"type":"extras","title":"Strict rendering - Overview","doc":"If there are any missing variables/filters and `strict_variables: true` or `strict_filters: true` are passed as options `Solid.render/3` returns `{:error, errors, result}` where errors is the list of collected errors and `result` is the rendered template.\n\n`Solid.render!/3` raises if `strict_variables: true` is passed and there are missing variables.\n`Solid.render!/3` raises if `strict_filters: true` is passed and there are missing filters.","ref":"readme.html#strict-rendering"},{"type":"extras","title":"Caching - Overview","doc":"In order to cache `render`-ed templates, you can write your own cache adapter. It should implement behaviour `Solid.Caching`. By default it uses `Solid.Caching.NoCache` trivial adapter.\n\nIf you want to use for example [Cachex](https://github.com/whitfin/cachex) for that such implemention would look like:\n\n```elixir\ndefmodule CachexCache do\n  @behaviour Solid.Caching\n\n  @impl true\n  def get(key) do\n    case Cachex.get(:your_cache_name, key) do\n      {_, nil} -> {:error, :not_found}\n      {:ok, value} -> {:ok, value}\n      {:error, error_msg} -> {:error, error_msg}\n    end\n  end\n\n  @impl true\n  def put(key, value) do\n    case Cachex.put(:my_cache, key, value) do\n      {:ok, true} -> :ok\n      {:error, error_msg} -> {:error, error_msg}\n    end\n  end\nend\n\n```\n\nAnd then pass it as an option to render `cache_module: CachexCache`. Now while using `{% render 'etc' %}` this custom cache will be used","ref":"readme.html#caching"},{"type":"extras","title":"Using structs in context - Overview","doc":"In order to pass structs to context you need to implement protocol `Solid.Matcher` for that. That protocol consist of one function `def match(data, keys)`. First argument is struct being provided and second is list of string, which are keys passed after `.` to the struct.\n\nFor example:\n\n```elixir\ndefmodule UserProfile do\n  defstruct [:full_name]\n\n  defimpl Solid.Matcher do\n    def match(user_profile, [\"full_name\"]), do: {:ok, user_profile.full_name}\n  end\nend\n\ndefmodule User do\n  defstruct [:email]\n\n  def load_profile(%User{} = _user) do\n    # implementation omitted\n    %UserProfile{full_name: \"John Doe\"}\n  end\n\n  defimpl Solid.Matcher do\n    def match(user, [\"email\"]), do: {:ok, user.email}\n    def match(user, [\"profile\" | keys]), do: user |> User.load_profile() |> @protocol.match(keys)\n  end\nend\n\ntemplate = ~s({{ user.email}}: {{ user.profile.full_name }})\ncontext = %{\n  \"user\" => %User{email: \"test@example.com\"}\n}\n\ntemplate |> Solid.parse!() |> Solid.render!(context) |> to_string()\n# => test@example.com: John Doe\n```\n\nIf the `Solid.Matcher` protocol is not enough one can provide a module like this:\n\n```elixir\ndefmodule MyMatcher do\n  def match(_data, _keys), do: {:ok, 42}\nend\n\n# ...\nSolid.render!(template, %{\"number\" => 4}, matcher_module: MyMatcher)\n```","ref":"readme.html#using-structs-in-context"},{"type":"extras","title":"Sigil Support - Overview","doc":"Solid provides a `~LIQUID` sigil for validating and compiling templates at compile time:\n\n```elixir\nimport Solid.Sigil\n\n# Validates syntax at compile time\ntemplate = ~LIQUID\"\"\"\nHello, {{ name }}!\n\"\"\"\n\n# Use the compiled template\nSolid.render!(template, %{\"name\" => \"World\"})\n```\n\nThe sigil will raise helpful CompileError messages with line numbers and context when templates contain syntax errors.\nExperimental VSCode syntax highlighting is available with the [Liquid Sigil](https://marketplace.visualstudio.com/items?itemName=JakubSkalecki.liquid-sigil) extension.","ref":"readme.html#sigil-support"},{"type":"extras","title":"Contributing - Overview","doc":"When adding new functionality or fixing bugs consider adding a new test case here inside `test/solid/integration/scenarios`. These scenarios are tested against the Ruby gem so we can try to stay as close as possible to the original implementation.","ref":"readme.html#contributing"},{"type":"extras","title":"Copyright and License - Overview","doc":"Copyright (c) 2016-2025 Eduardo Gurgel Pinho\n\nThis work is free. You can redistribute it and/or modify it under the\nterms of the MIT License. See the [LICENSE.md](./LICENSE.md) file for more details.","ref":"readme.html#copyright-and-license"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.2"}}